import pandas as pd

from normalisation import constants


def compute_sum_by_fragments(raw_df):
    """
    Add intensities for all fragments corresponding to one
    unlabeled fragment.

    :param raw_df: Dataframe containing intensities of fragment for each
                   metabolite across different samples
    :return: Dataframe with added intensities of fragment for each metabolite
             within the sample
    """
    intensity_sum = \
        raw_df.groupby([constants.MQ_SAMPLE_NAME, constants.MQ_COHORT_NAME,
                        constants.PARENT])[
            constants.MQ_INTENSITY].sum().reset_index()
    del raw_df[constants.MQ_INTENSITY]
    raw_df = intensity_sum.merge(raw_df, how="inner",
                          on= (constants.MQ_SAMPLE_NAME,
                          constants.MQ_COHORT_NAME,
                          constants.PARENT))
    raw_df.reset_index(
        level=[constants.PARENT, constants.MQ_SAMPLE_NAME], inplace=True,
        drop=True)
    raw_df = raw_df.drop_duplicates(
        subset=[constants.PARENT, constants.MQ_SAMPLE_NAME])

    raw_df[constants.MQ_CONCENTRATION] = pd.Series(-1, raw_df.index)
    return raw_df


def split_std_bio(raw_df, metadata_std):
    """
    Divide the raw df into standard and biosample dataframe

    :param raw_df: Dataframe that contains intensities of each fragment
                   present in the samples
    :param metadata_std: Standard metadata dataframe containing concentration
                        mapping of the standard samples
    :return: Standard and biosample dataframes that contain intensities of
             standard and samples with unknown concentration respectively.
    """
    std_sample_list = metadata_std[constants.MQ_SAMPLE_NAME].unique()
    standard_df = raw_df[raw_df[constants.MQ_SAMPLE_NAME].isin(
        std_sample_list)]
    if standard_df.empty:
        raise ValueError(
            'Std samples not found in ' + constants.MQ_COHORT_NAME + ' column')
    biosample_index = raw_df.index.difference(standard_df.index)
    biosamples_df = raw_df.ix[biosample_index, :]
    return standard_df, biosamples_df


def merge_std_df_metadata(metadata_std, standard_df):
    """
    Merge concentration mappings to standard dataframe

    :param metadata_std: standard metadata dataframe containing concentration
                        mapping of the standard samples
    :param standard_df: standard dataframe with intensities values for
                        standard samples
    :return: Merge standard dataframe having concentration mapped with sample
             intensities
    """
    del standard_df[constants.MQ_CONCENTRATION]

    merged_std = standard_df.merge(metadata_std, how='inner',
                                   on=[constants.MQ_SAMPLE_NAME,
                                       constants.MQ_COHORT_NAME,
                                       constants.PARENT])
    merged_std = merged_std.drop_duplicates().reset_index(drop=True)
    if merged_std.empty:
        raise Exception(
            'No common entries for ' + constants.MQ_SAMPLE_NAME + ','
            + constants.MQ_COHORT_NAME + ' or ' + constants.PARENT +
            ' in metadata_std_sample and raw files.')
    return merged_std


def merge_mq_metadata(raw_mq, metadata_mq):
    """
    Merge the mapping of fragments to unlabeled fragment within the raw mq
    dataframe

    :param raw_mq: dataframe having fragment along with intensties values
    :param metadata_mq: dataframe having mapping of unlabeled fragment with
                        metabolite fragment
    :return: Raw dataframe with merged unlabeled fragment for corresponding
            metabolite.
    """
    try:
        assert set(metadata_mq[constants.PARENT]).issubset(
            set(metadata_mq[constants.MQ_FRAGMENT]))
        raw_mq = raw_mq.merge(metadata_mq, how='inner',
                              on=constants.MQ_FRAGMENT)
        if raw_mq.empty:
            raise Exception(
                'No common entries for ' + constants.MQ_FRAGMENT +
                ' in metadata_mq and raw files.')
        raw_mq.fillna(0, inplace=True)
        raw_mq  = raw_mq.drop_duplicates()
    except KeyError:
        raise KeyError('Missing columns: ' + constants.MQ_FRAGMENT)
    except AssertionError:
        raise AssertionError(
            constants.MQ_FRAGMENT + ' must be subset of ' + constants.PARENT)
    return raw_mq